# Lab 2

## Exercise 1 (Text entropy)

The script `entropy` expects a text file as its argument and will calculate the
token-based entropy of the text after splitting it on whitespace. Usage example:

```shell
$ python3 entropy.py all-the-news.txt
Calculating entropy based on 212690 tokens on 10000 lines.
Entropy: 10.055453826580258
```

The file `test_entropy.py` contains some simple unit tests for the entropy
calculation. It can be executed without arguments. Make sure that the
`entropy.py` module is importable, e.g. by executing `test_entropy.py`
from the directory `edit_distance.py` resides in or setting the `PYTHONPATH`
appropriately.

```shell
$ PYTHONPATH=/path/to/dir/with/entropy.py python3 test_entropy.py
...
----------------------------------------------------------------------
Ran 3 tests in 0.000s

OK
```

Output for DSTC2:
```
Calculating entropy based on 131813 tokens on 10000 lines.
Entropy: 6.747774616064247
```

Output for Babi:
```
Calculating entropy based on 103221 tokens on 10000 lines.  
Entropy: 5.9655501329263085                                                
```

Output for All the News:
```
Calculating entropy based on 212690 tokens on 10000 lines.
Entropy: 10.055453826580258         
```

The Dialog bAbI tasks dataset is made up of short auto-generated dialogs in the
restaurant domain, more specifically a user wants a system to find a restaurant
and make a reservation there. The DSTC2 dataset contains dialog logs of
dialogs, apparently collected via Amazon Mechanical Turk, but only the
utterances of the system can be easily extracted as a string, if I’m not
mistaken. The news dataset contains news articles from 15 American news
outlets. The news dataset has a significantly larger entropy than the other two
datasets, which is to be expected because it is (1) multiple-domain, (2)
written by actual huamns and (3) written not to be as short or exact as
possible but rather to be pleasant to read. DSTC2 has a slightly higher entropy
than the bAbI dataset, which might be due to the former including more low
frequency elements like phone numbers and addresses.

The extraction was done using the three scripts `extract_dstc2.sh`,
`extract_babi.sh` and `extract_all_the_news.py`.


## Exercise 2 (Language Model Cross Entropy)

I used the 2016 Czech Vystadial data and gathered the content of all the small
files into bigger train and dev files using the script `extract_vystadial.sh`,
which also removes non-speech tokens such as `_INHALE_`. I then trained a
5-gram model on the train part according to the instructions.

I calculated the cross entropy of the model with the `ken_crossentropy.py`
script which depends on the Python module `kenlm` being installed. Installation
instructions are [here](https://github.com/kpu/kenlm#installation).  

Here are the results for the training set, the dev set, the first sentence of
the dev set („NO DAŘÍ NO“) and the seventh sentence of the dev set („CHCI JET
ZE ZASTÁVKY KAČEROV DO ZASTÁVKY SÍDLIŠTĚ PÍSNICE“):

```shell
$ python3 ken_crossentropy.py vystadial_cz_2016.arpa vystadial_cz_2016_train.txt
Sentence Count: 95409
Token Count: 431943
Cross Entropy: 1.2759683188597328
Unigram Cross Entropy: 4.25219862344923
```

```shell
$ python3 ken_crossentropy.py vystadial_cz_2016.arpa vystadial_cz_2016_dev.txt
Sentence Count: 1741
Token Count: 7810
Cross Entropy: 2.2602838986259775
Unigram Cross Entropy: 4.25340459694783
```

```bash
$ python3 ken_crossentropy.py vystadial_cz_2016.arpa <(head -n 1 vystadial_cz_2016_train.txt)
Sentence Count: 1
Token Count: 3
Cross Entropy: 3.2480398615201316
Unigram Cross Entropy: 3.33508030573527
```

```bash
$ python3 ken_crossentropy.py vystadial_cz_2016.arpa <(echo 'CHCI JET ZE ZASTÁVKY KAČEROV DO ZASTÁVKY SÍDLIŠTĚ PÍSNICE')
Sentence Count: 1
Token Count: 9
Cross Entropy: 0.7542210223360194
Unigram Cross Entropy: 3.996779282887777
```

With the two whole dataset results, it can be noticed that the cross entropy is
smaller when the whole five-gram model can be used than the cross entropy of a
model depending only on unigram frequencies. This is to be expected because
this way, tokens are assigned a higher probability if they occur in a similar
context as they do in the training set. The cross entropy is higher on the
training set than on the dev set, which also is to be expected because the
probabilities very estimated based on the training set. The unigram cross
entropy, however, is very similar for the two sets which suggests that overall
the two sets exhibit very similar unigram distributions.

When calculating the cross entropy for single sentences, the cross entropy
naturally has a higher variance than for whole datasets. Some sentences (or
rather word sequences) might be rather unusual (like „NO DAŘÍ NO“) resulting in
a cross entropy higher than for a whole corpus, while others are perfectly
natural (like „CHCI JET ZE ZASTÁVKY KAČEROV DO ZASTÁVKY SÍDLIŠTĚ PÍSNICE“),
which is reflected in a very low cross entropy.
