#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import argparse
from collections import Counter
import math
from typing import Sequence


def get_entropy(tokens: Sequence[str]) -> float:
    """Calculate the entropy of a sequence of strings."""
    if not tokens:
        return None

    counter = Counter(tokens)
    total = len(tokens)
    entropy = - sum(counter[token]/total * math.log2(counter[token]/total)
                    for token in set(tokens))
    return entropy


def main(textfile, limit):
    """Read the first `limit` lines of a textfile and calculate its entropy."""
    lines = []
    with open(textfile) as f:
        for i, line in enumerate(f):
            if i < limit:
                lines.append(line)
            else:
                break
    tokens = [token for line in lines for token in line.split()]
    print('Calculating entropy based on {} tokens on {} lines.'
          .format(len(tokens), len(lines)))
    entropy = get_entropy(tokens)
    print('Entropy: {}'.format(entropy))


def parse_args():
    description = 'Calculate word-based entropy for a given text.'
    parser = argparse.ArgumentParser(description=description)
    parser.add_argument('--limit', '-l', type=int, default=10000)
    parser.add_argument('textfile')
    args = parser.parse_args()
    return args


if __name__ == '__main__':
    ARGS = parse_args()
    main(**vars(ARGS))
