#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import argparse
import csv
import re

# Python’s re does not support unicode character classes …
# So just split on some random punctuation marks I came up with.
PUNCTUATION_REGEX = r'(\.|\?|!|:|,|;|‚|‘|’|„|“|’|"|\')'


def tokenize_and_sentence_split(article):
    """Naively split a text into sentences and tokens."""
    sentences = re.split(r'\.|\?|!', article)
    result_sentences = []
    for sentence in sentences:
        sentence = re.subn(PUNCTUATION_REGEX, r' \1 ', sentence.strip())[0]
        sentence = re.subn(r'\s+', ' ', sentence)[0]
        sentence = sentence.lower()
        result_sentences.append(sentence)
    return result_sentences


def extract_content(csvfile, outfile, delimiter=',', quotechar='"',
                    content_column=9):
    with open(csvfile, newline='') as infile, open(outfile, 'w') as out:
        reader = csv.reader(infile, delimiter=delimiter, quotechar=quotechar)
        i = 1
        for row in reader:
            for utterance in tokenize_and_sentence_split(row[content_column]):
                print(utterance, file=out)


def main(articles, outfile):
    # Line 17535 contains 153205 bytes which is too much for the default
    # field limit.
    csv.field_size_limit(int(1e6))
    for csvfile in articles:
        extract_content(csvfile, outfile)


def parse_args():
    description = 'Extract content from All the News articles'
    parser = argparse.ArgumentParser(description=description)
    parser.add_argument('--outfile', '-o', default='/dev/stdout')
    parser.add_argument('articles', nargs='+')
    args = parser.parse_args()
    return args


if __name__ == '__main__':
    ARGS = parse_args()
    main(**vars(ARGS))
