#!/usr/bin/env bash

while getopts ':hsu' opt; do
    case $opt in
        h)
            GET_HELP=true
            ;;
        s)
            EXCLUDE_SILENCE=true
            ;;
        u)
            EXCLUDE_UNDERSCORES=true
            ;;
        \?)
            err_with_help "Invalid option: -$OPTARG"
            ;;
    esac
done
shift $((OPTIND-1))

BABI_TOP_DIR="$1"
OUTFILE="$2"

usage() {
    echo 'Usage: extract_dstc2.sh [-s] [-u] BABI_TOP_DIR OUTFILE'
}

if [ -n "$GET_HELP" ]; then
    usage
    exit 0
elif [ -z "$BABI_TOP_DIR" -o -z "$OUTFILE" ]; then
    usage
    exit 1
elif [ ! -d "$BABI_TOP_DIR" ]; then
    echo "No such directory: $BABI_TOP_DIR"
    usage
    exit 2
fi

if [ -n "$EXCLUDE_UNDERSCORES" ]; then
    LINE_FILTER='/_/! '
fi

find "$BABI_TOP_DIR" -name 'dialog*.txt' | \
    while read -r fname; do
        sed -n "$LINE_FILTER"' s/^[[:digit:]]\+\s\+\(.*\)$/\1/p' "$fname" | awk '{ print tolower($0); }'
    done >"$OUTFILE"

if [ -n "$EXCLUDE_SILENCE" ]; then
    sed -i 's/<silence>//' "$OUTFILE"
fi

# vim: sts=4 sw=4 et si
