# Lab 3

## Exercise 1 (DSTC2 Dialogue State Tracking)

The code for this exercise lives in the repository
[Simon-Will/ds-dstc2](https://www.gitlab.com/Simon-Will/ds-dstc2) at
gitlab.com. Apart from some smaller changes in the provided scripts,
the main additions to the upstream repository are the addition of two
models (a simple regex-based model as well as a CNN-RNN-based model)
and the addition of an evaluation module.

### The Regex Model

The regex model in `simpe_match_model.py` concatenates all the
previous utterances by the user and tries to find literal matches for
the food, area and pricerange labels provided in the ontology. If it
finds more than one match for a given category, the most recent match
is used. If no match is found, the model predicts the “none” class.

### The CNN-RNN Model

The neural network model in `cnn_rnn_will.py` employs a single CNN
layer to every turn in the dialogue in a similar vein as Kim’s (2016)
ACL paper about “Convolutional Neural Networks for Sentence
Classification”. Afterwards, it uses a GRU to process the resulting
turn vectors sequentially. Three different projection layers are then
used on the GRU’s last hidden unit to predict the labels.

It’s not really written to be run on the GPU: Pytorch tensor are not
allocated on the GPU and only a batch size of 1 is supported.

### Evaluation

The module `evaluation.py` is used to measure precision, recall,
F1-measure and accuracy for the single labels as well as the overall
accuracy for the joint labels.

### Results

The regex model achieved an accuracy of 49 %. The single scores differ
considerably between each other. The food precision is very high
because if a cuisine like “italian” is mentioned in one of these
utterances it is very unlikely that it’s referring to something other
than food. For the area, however, it’s not that easy: When an
utterance mentions “north american”, it will refer to the North
American cuisine in this dataset, but the regex model will still
predict “north” for the area, anyway. This would be actually easy to
improve.

```
$ PYTHONPATH=. tracker/simple_match_model_will.py
*** joint_turn_accuracy: 0.49 ***
area_turn_accuracy: 0.75
area_turn_f1: 0.78
area_turn_precision: 0.94
area_turn_recall: 0.66
food_turn_accuracy: 0.79
food_turn_f1: 0.84
food_turn_precision: 0.98
food_turn_recall: 0.74
pricerange_turn_accuracy: 0.89
pricerange_turn_f1: 0.90
pricerange_turn_precision: 0.98
pricerange_turn_recall: 0.83
```

Unfortunately, the CNN RNN model didn’t produce any sensible results
and I don’t have the time to debug it. The log file `train.log` in the
`ds-dstc2` repo shows that the training log did go down at least.

```
*** joint_turn_accuracy: 0.00 ***
area_turn_accuracy: 0.00
area_turn_f1: 0.00
area_turn_precision: 0.00
area_turn_recall: 0.00
food_turn_accuracy: 0.16
food_turn_f1: 0.11
food_turn_precision: 0.13
food_turn_recall: 0.10
pricerange_turn_accuracy: 0.00
pricerange_turn_f1: 0.00
pricerange_turn_precision: 0.00
pricerange_turn_recall: 0.00
```

## Exercise 2 (BLEU score)

The script `bleu_demo` uses the BLEU implementation from
https://github.com/tensorflow/nmt/blob/master/nmt/scripts/bleu.py to
show a couple of shortcomings of the BLEU score. Here is its output:

```
Reference: Hey James , what are you going to do today ?

System 1: Hi James , what are you planning to do today ?
BLEU score: 0.8128081610557882

System 2: Hi James , do you already have any plans for today ?
BLEU score: 0.4466343020620244

System 3: to do today ? Hey James , what are you going
BLEU score: 0.9454469089074679
```

The first system generates a sentence that is virtually identical in
meaning and also close to the structure of the reference. It is
rewarded with a high BLEU score, but it is incorrectly penalized for
the swaps of the words “Hi” and “going”.

The second system conveys the same meaning as the reference and is
idiomatic English, as well. However, the wording is very different
from the reference, which is why the BLEU score is not that high.

The third system produces an incoherent string of words, but sports
the highest BLEU score because it has many n-gram overlaps even though
it is obviously the worst candidate.