#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import json
import os

from parlai.core.teachers import DialogTeacher
from .build import build


def _path(opt):
    build(opt)

    datatype = opt['datatype'].split(':')[0]
    fname = 'data.dstc2.{}.json'.format(datatype)
    path = os.path.join(opt['datapath'], 'dstc2', fname)
    return path


def _generate_possible_labels(opt):
    build(opt)
    ontology_file = os.path.join(opt['datapath'], 'dstc2',
                                 'data.dstc2.ontology.json')
    with open(ontology_file) as f:
        ontology = json.load(f)

    informable = ontology['informable']
    for food in informable['food']:
        for area in informable['area']:
            for pricerange in informable['pricerange']:
                yield '{} {} {}'.format(food, area, pricerange)


class DSTC2Teacher(DialogTeacher):

    def __init__(self, opt, shared=None):
        self.datatype = opt['datatype'].split(':')[0]
        self.id = 'dstc2'
        #print(os.listdir(os.path.join(opt['datapath'], 'dstc2')))

        self._label_candidates = list(_generate_possible_labels(opt))
        opt['datafile'] = _path(opt)

        super().__init__(opt, shared)

    def setup_data(self, path):
        with open(path) as f:
            dialogues = json.load(f)

        for dialogue in dialogues:
            num_turns = len(dialogue)
            for i, turn in enumerate(dialogue):
                user_utterance = turn[1]
                label = turn[4]
                episode_done = i == num_turns - 1
                yield (user_utterance, [label], None, None, None), episode_done

    def label_candidates(self):
        return self._label_candidates


class DefaultTeacher(DSTC2Teacher):
    pass
