#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import os

import parlai.core.build_data as build_data


def build(opt):
    dpath = os.path.join(opt['datapath'], 'dstc2')

    if not build_data.built(dpath):
        print('[building data: {}'.format(dpath))

        build_data.make_dir(dpath)

        url_root = 'https://gitlab.com/ufal/dsg/ds-dstc2/raw/master/data/dstc2/'
        fname_pattern = 'data.dstc2.{}.json'
        for datatype_orig, datatype_parlai in (('train', 'train'), ('dev', 'valid'),
                                               ('test', 'test')):
            url = url_root + fname_pattern.format(datatype_orig)
            fname = fname_pattern.format(datatype_parlai)
            build_data.download(url, dpath, fname)

        ontology_fname = 'data.dstc2.ontology.json'
        ontology_url = url_root + ontology_fname
        build_data.download(ontology_url, dpath, ontology_fname)

        build_data.mark_done(dpath)
